# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Usage: copyto [-rpC] -c CLUSTER [-u USER] FILE ... REMOTE_DIR

Copies FILEs to REMOTE_DIR on each node of the specified CLUSTER. If CLUSTER
is omitted, then the default cluster is used. CLUSTER may also name
a single node.

REMOTE_DIR must be an absolute path, (i.e. must begin with '/').

The flags are a subset of the options supported by scp:
-r: recursive copy
-p: preserve modification times, access times, and modes.
-C: enable compression.
"""

import pickle
import popen2
import threading

import osh.oshobjects

def copyto(*args):
    if len(args) > 0:
        cluster = args[0]
        args = (osh.oshargs.Option('-c', cluster),) + args[1:]
    return _CopyTo().process_args(*args)

class _CopyTo(osh.oshobjects.RemoteOp):

    # state

    _remote_dir = None
    _files = None
    _scp_options = None


    # object interface
    
    def __init__(self):
        osh.oshobjects.RemoteOp.__init__(self, 'rpCc:u:', (2, None))


    # BaseOp interface
    
    def doc(self):
        return __doc__

    def setup(self):
        osh.oshobjects.RemoteOp.setup(self)
        args = self._args.remaining()
        if len(args) < 2:
            self.usage()
        self._remote_dir = args[-1]
        if not self._remote_dir.startswith('/'):
            self.usage()
        self._files = args[:-1]
        for option in osh.oshobjects.SCP_OPTIONS:
            self._check_option(option)


    # RemoteOp interface

    def action(self, host):
        osh.oshobjects.copyup(self._files,
                              self.user(),
                              host.address(),
                              self._remote_dir,
                              self.scp_options())
