# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Usage: f FUNCTION
    
Receives a stream of objects through stdin, applies FUNCTION to each,
and passes the resulting object to stdout. For example, if stdin
contains ints, then we can negate each int as follows:

... ^ f 'x: -x' ^ ...
"""

import types

import osh.oshfunction
import osh.oshobjects

# f can be used as a generator (function with no args) or
# downstream. That's why receive and execute are both defined.

def f(*args):
    return _F().process_args(*args)

class _F(osh.oshobjects.Generator):

    _oshfunction = None


    # object interface

    def __init__(self):
        osh.oshobjects.Generator.__init__(self, '', (1, 1))


    # BaseOp interface
    
    def doc(self):
        return __doc__

    def setup(self):
        args = self.args()
        self._oshfunction = args.next_function()
        if self._oshfunction is None or args.has_next():
            self.usage()

    def receive(self, object):
        self.send(self._oshfunction(object))


    # Generator interface

    def execute(self):
        self.send(self._oshfunction())
        self.send_complete()
