# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Usage: sort [KEY_FUNCTION]

If KEY_FUNCTION is not provided then objects from stdin are sorted
using the default sorting (i.e., using the cmp function applied to
the objects being sorted).

If KEY_FUNCTION is provided, then sorting is done by applying cmp to
keys obtained by applying KEY_FUNCTION to the input objects.

Example: Suppose stdin contains a set of Person objects. To sort by
Person.age() and resolve ties by Person.height(), do the following:

... ^ sort 'p: (p.age(), p.height())' ^ ...
"""

import osh.oshfunction
import osh.oshobjects

def sort(*args):
    return _Sort().process_args(*args)

class _Sort(osh.oshobjects.Op):

    _key = None
    _list = None


    # object interface

    def __init__(self):
        osh.oshobjects.Op.__init__(self, '', (0, 1))


    # BaseOp interface
    
    def doc(self):
        return __doc__

    def setup(self):
        args = self.args()
        if args.has_next():
            self._key = args.next_function()
        if args.has_next():
            self.usage()
        self._list = []
    
    def receive(self, object):
        self._list.append(object)
    
    def receive_complete(self):
        key = self._key
        if key:
            # Attach the keys, sort by the key, then detach the key.
            self._list = [(key(object), object) for object in self._list]
            self._list.sort(lambda x, y: cmp(x[0], y[0]))
            self._list = [t[1] for t in self._list]
        else:
            self._list.sort()
        for object in self._list:
            self.send(object)
