# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

def _type_and_attribute(x):
    open = x.find('(')
    if open == -1:
        type = None
        attribute = x.strip()
    else:
        type = x[:open].strip()
        close = x.find(')')
        attribute = x[open + 1 : close].strip()
    return type, attribute

class AttributeMap:
    _attributes = None
    _conversions = None
    _positions = None

    def __init__(self, *typedAttributes):
        self._attributes = []
        self._conversions = []
        self._positions = {}
        position = 0
        for x in typedAttributes:
            type, attribute = _type_and_attribute(x)
            self._positions[attribute] = position
            position += 1
            self._attributes.append(attribute)
            if type:
                conversion = eval('lambda x: %s(x)' % type)
            else:
                conversion = None
            self._conversions.append(conversion)

    def attributes(self):
        return self._attributes

    def conversions(self):
        return self._conversions

    def position(self, attribute):
        return self._positions.get(attribute, None)

class OshStruct:
    _state = None

    def __init__(self, state):
        self._state = state

    def __getattr__(self, attribute):
        attribute_map = self.attribute_map()
        position = attribute_map.position(attribute)
        if position is None:
            raise AttributeError, ('%s has no attribute %s' %
                                   (str(self.__class__), attribute))
        else:
            attributeValue = self._state[position]
            conversion = attribute_map.conversions()[position]
            if conversion:
                attributeValue = conversion(attributeValue)
        return attributeValue

    def __repr__(self):
        attribute_map = self.attribute_map()
        description = [str(self.__class__.__name__), '{']
        for attribute in attribute_map.attributes():
            position = attribute_map.position(attribute)
            if position > 0:
                description += ', '
            description += attribute
            description += ':'
            description += str(self._state[position])
        description += '}'
        return ''.join(description)

    def __str__(self):
        return self.__repr__()

    def attribute_map(self):
        assert false
